@@ -2,6 +2,8 @@ import warnings
 
 import delayedarray
 import numpy
+import dask.array
+import scipy.sparse
 
 
 def test_DelayedArray_dense():
@@ -17,6 +19,10 @@ def test_DelayedArray_dense():
     assert isinstance(dump, numpy.ndarray)
     assert (dump == raw).all()
 
+    da = delayedarray.create_dask_array(x)
+    assert isinstance(da, dask.array.core.Array)
+    assert (dump == da.compute()).all()
+
 
 def test_DelayedArray_isometric_add():
     test_shape = (55, 15)
@@ -67,6 +73,10 @@ def test_DelayedArray_isometric_add():
     assert z.seed.right.shape == test_shape
     assert (numpy.array(z) == y + y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_subtract():
     test_shape = (55, 15)
@@ -101,6 +111,10 @@ def test_DelayedArray_isometric_subtract():
     assert z.shape == x.shape
     assert (numpy.array(z) == y - y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_multiply():
     test_shape = (35, 25)
@@ -135,6 +149,10 @@ def test_DelayedArray_isometric_multiply():
     assert z.shape == x.shape
     assert (numpy.array(z) == y - y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_divide():
     test_shape = (35, 25)
@@ -169,6 +187,10 @@ def test_DelayedArray_isometric_divide():
     assert z.shape == x.shape
     assert (numpy.array(z) == y / y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_modulo():
     test_shape = (22, 44)
@@ -203,6 +225,10 @@ def test_DelayedArray_isometric_modulo():
     assert z.shape == x.shape
     assert (numpy.array(z) == y % y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_floordivide():
     test_shape = (30, 55)
@@ -237,6 +263,10 @@ def test_DelayedArray_isometric_floordivide():
     assert z.shape == x.shape
     assert (numpy.array(z) == y // y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_power():
     test_shape = (30, 55)
@@ -273,6 +303,10 @@ def test_DelayedArray_isometric_power():
     assert z.shape == x.shape
     assert (numpy.array(z) == y**y2).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_equal():
     test_shape = (30, 55, 10)
@@ -300,6 +334,10 @@ def test_DelayedArray_isometric_equal():
     assert z.shape == x.shape
     assert (numpy.array(z) == (y == y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_not_equal():
     test_shape = (12, 42)
@@ -327,6 +365,10 @@ def test_DelayedArray_isometric_not_equal():
     assert z.shape == x.shape
     assert (numpy.array(z) == (y != y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_greater():
     test_shape = (42, 11)
@@ -354,6 +396,10 @@ def test_DelayedArray_isometric_greater():
     assert z.shape == x.shape
     assert (numpy.array(z) == (y > y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_greater_equal():
     test_shape = (24, 13)
@@ -381,6 +427,10 @@ def test_DelayedArray_isometric_greater_equal():
     assert z.shape == x.shape
     assert (numpy.array(z) == (y >= y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_less():
     test_shape = (24, 13)
@@ -408,6 +458,10 @@ def test_DelayedArray_isometric_less():
     assert z.shape == x.shape
     assert (numpy.array(z) == (y < y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_less_than():
     test_shape = (14, 33)
@@ -435,6 +489,10 @@ def test_DelayedArray_isometric_less_than():
     assert z.shape == x.shape
     assert (numpy.array(z) == (y <= y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_logical_and():
     test_shape = (23, 33)
@@ -462,6 +520,10 @@ def test_DelayedArray_isometric_logical_and():
     assert z.shape == x.shape
     assert (numpy.array(z) == numpy.logical_and(y, y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_logical_or():
     test_shape = (23, 55)
@@ -489,6 +551,10 @@ def test_DelayedArray_isometric_logical_or():
     assert z.shape == x.shape
     assert (numpy.array(z) == numpy.logical_or(y, y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_logical_xor():
     test_shape = (44, 55)
@@ -516,6 +582,10 @@ def test_DelayedArray_isometric_logical_xor():
     assert z.shape == x.shape
     assert (numpy.array(z) == numpy.logical_xor(y, y2)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_isometric_simple():
     test_shape = (30, 55)
@@ -564,6 +634,7 @@ def test_DelayedArray_isometric_simple():
             ufunc = getattr(numpy, op)
             z = ufunc(x)
             obs = numpy.array(z)
+            da = delayedarray.create_dask_array(z).compute()
             expected = ufunc(expanded)
 
         assert isinstance(z, delayedarray.DelayedArray)
@@ -572,9 +643,12 @@ def test_DelayedArray_isometric_simple():
 
         missing = numpy.isnan(obs)
         assert (missing == numpy.isnan(expected)).all()
+        assert (missing == numpy.isnan(da)).all()
         obs[missing] = 0
         expected[missing] = 0
+        da[missing] = 0
         assert (obs == expected).all()
+        assert (obs == da).all()
 
 
 def test_DelayedArray_subset():
@@ -624,6 +698,11 @@ def test_DelayedArray_subset():
     stuff = x[:, :, 2]
     assert (stuff == y[:, :, 2]).all()
 
+    # Works with dask arrays.
+    da = delayedarray.create_dask_array(x)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(x) == da.compute()).all()
+
 
 #    # Trying vectorized index.
 #    stuff = x[[1,2,3],[4,5,6],[7,8,9]]
@@ -653,6 +732,10 @@ def test_DelayedArray_combine():
     assert x.seed.along == 1
     assert (numpy.array(x) == numpy.concatenate((y1.seed, y2.seed), axis=1)).all()
 
+    da = delayedarray.create_dask_array(x)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(x) == da.compute()).all()
+
 
 def test_DelayedArray_transpose():
     y = numpy.random.rand(30, 23)
@@ -682,6 +765,11 @@ def test_DelayedArray_transpose():
     assert t.shape == (10, 23, 30)
     assert (numpy.array(t) == numpy.transpose(y)).all()
 
+    # Works with dask arrays.
+    da = delayedarray.create_dask_array(t)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(t) == da.compute()).all()
+
 
 def test_DelayedArray_cast():
     y = numpy.random.rand(30, 23) * 10
@@ -693,6 +781,10 @@ def test_DelayedArray_cast():
     assert z.shape == (30, 23)
     assert (numpy.array(z) == y.astype(numpy.int32)).all()
 
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
 
 def test_DelayedArray_round():
     y = numpy.random.rand(30, 23) * 10
@@ -708,3 +800,39 @@ def test_DelayedArray_round():
     # Number of places.
     z = numpy.round(x, decimals=1)
     assert (numpy.array(z) == numpy.round(y, decimals=1)).all()
+
+    da = delayedarray.create_dask_array(z)
+    assert isinstance(da, dask.array.core.Array)
+    assert (numpy.array(z) == da.compute()).all()
+
+
+def test_DelayedArray_sparse():
+    y = scipy.sparse.csr_matrix([[1, 2, 0], [0, 0, 3], [4, 0, 5]])
+    x = delayedarray.DelayedArray(y)
+
+    out = delayedarray.extract_array(x)
+    assert isinstance(out, numpy.ndarray) is False
+
+    z = x + 1
+    out = delayedarray.extract_array(z)
+    assert isinstance(out, numpy.ndarray) is True
+
+    v = numpy.random.rand(3)
+    z = x * v
+    out = delayedarray.extract_array(z)
+    assert (y.toarray() * v == numpy.array(out)).all()
+
+    z = x[1:3, [0, 2]]
+    out = delayedarray.extract_array(z)
+    assert isinstance(out, numpy.ndarray) is False
+    assert (y.toarray()[1:3, [0, 2]] == out.toarray()).all()
+
+    z = numpy.log1p(x)
+    out = delayedarray.extract_array(z)
+    assert isinstance(out, numpy.ndarray) is False
+    assert (numpy.log1p(y.toarray()) == out.toarray()).all()
+
+    z = numpy.transpose(x)
+    out = delayedarray.extract_array(z)
+    assert isinstance(out, numpy.ndarray) is False
+    assert (numpy.transpose(y.toarray()) == out.toarray()).all()
